@@ -41,6 +41,7 @@ import util
 
 try:
     import hildon
+    from portrait import FremantleRotation
 except:
     if util.platform == util.MAEMO:
         log.critical( 'Using GTK widgets, install "python2.5-hildon" '
@@ -53,7 +54,7 @@ from dbusinterface import interface
 
 about_name = 'Panucci'
 about_text = _('Resuming audiobook and podcast player')
-about_authors = ['Thomas Perl', 'Nick (nikosapi)', 'Matthew Taylor']
+about_authors = ['Thomas Perl', 'nikosapi', 'Matthew Taylor']
 about_website = 'http://panucci.garage.maemo.org/'
 app_version = ''
 donate_wishlist_url = 'http://www.amazon.de/gp/registry/2PD2MYGHE6857'
@@ -68,12 +69,18 @@ coverart_size = [240, 240] if util.platform == util.MAEMO else [130, 130]
 gtk.about_dialog_set_url_hook(util.open_link, None)
 gtk.icon_size_register('panucci-button', 32, 32)
 
-def image(widget, filename, is_stock=False):
+def image2(widget, filename, is_stock=False, is_name=False):
     widget.remove(widget.get_child())
+    image(widget, filename, is_stock, is_name)
+
+def image(widget, filename, is_stock=False, is_name=False):
     image = None
     if is_stock:
         image = gtk.image_new_from_stock(
             filename, gtk.icon_size_from_name('panucci-button') )
+    elif is_name:
+        image = gtk.image_new_from_icon_name(
+            filename, gtk.icon_size_from_name('panucci-button') )
     else:
         filename = util.find_image(filename)
         if filename is not None:
@@ -168,7 +175,7 @@ class BookmarksWindow(hildon.StackableWindow):
         self.vbox = gtk.VBox()
         self.vbox.set_spacing(5)
         self.treeview = gtk.TreeView()
-        self.treeview.set_headers_visible(True)
+        self.treeview.set_headers_visible(False)
 
         # The tree lines look nasty on maemo
         if util.platform == util.LINUX:
@@ -178,9 +185,12 @@ class BookmarksWindow(hildon.StackableWindow):
         ncol = gtk.TreeViewColumn(_('Name'))
         ncell = gtk.CellRendererText()
         ncell.set_property('editable', True)
+	import pango
+        ncell.set_property('ellipsize', pango.ELLIPSIZE_END)
         ncell.connect('edited', self.label_edited)
         ncol.pack_start(ncell)
         ncol.add_attribute(ncell, 'text', 1)
+	ncol.set_expand(True)
 
         tcol = gtk.TreeViewColumn(_('Position'))
         tcell = gtk.CellRendererText()
@@ -201,10 +211,13 @@ class BookmarksWindow(hildon.StackableWindow):
         self.treeview.enable_model_drag_dest(
             treeview_targets, gtk.gdk.ACTION_COPY )
 
-        sw = gtk.ScrolledWindow()
-        sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
-        sw.set_shadow_type(gtk.SHADOW_IN)
+        #sw = gtk.ScrolledWindow()
+        #sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
+        #sw.set_shadow_type(gtk.SHADOW_IN)
+	sw = hildon.PannableArea()
         sw.add(self.treeview)
+	self.treeview.set_property('hildon-ui-mode', 'edit')
+	self.treeview.get_selection().set_mode(gtk.SELECTION_SINGLE)
         self.vbox.add(sw)
 
         app_menu = hildon.AppMenu()
@@ -363,7 +376,7 @@ class GTK_Main(object):
 
         self.recent_files = []
         self.progress_timer_id = None
-        self.volume_timer_id = None
+        #self.volume_timer_id = None
         self.has_coverart = False
         self.make_main_window()
         self.bookmarks_window_open = False
@@ -429,11 +442,11 @@ class GTK_Main(object):
             gtk_box_reparent(buttonbox, self.forward_button, True, True)
             gtk_box_reparent(buttonbox, self.fforward_button, True, True)
             gtk_box_reparent(buttonbox, self.bookmarks_button, True, True)
-            gtk_box_reparent(buttonbox, self.volume_button, True, True)
+            #gtk_box_reparent(buttonbox, self.volume_button, True, True)
             main_vbox.pack_start(buttonbox, False, False)
 
             for child in self.main_hbox.get_children():
-                if child != main_vbox and child != self.volume:
+		if child != main_vbox: #and child != self.volume:
                     self.main_hbox.remove(child)
         else:
             main_vbox = gtk.VBox()
@@ -449,7 +462,7 @@ class GTK_Main(object):
 
             # Horizontal layout: all buttons in the buttonbox
             buttontbl = gtk.Table(2, 4, True)
-            buttontbl.set_size_request(-1, 160)
+            buttontbl.set_size_request(-1, 210)
 
             def gtk_table_reparent(t, w, left, right, top, bottom):
                 b = gtk.HBox()
@@ -460,8 +473,8 @@ class GTK_Main(object):
                     b.add(w)
 
             gtk_table_reparent(buttontbl, self.bookmarks_button, 0, 1, 0, 1)
-            gtk_table_reparent(buttontbl, self.play_pause_button, 1, 3, 0, 1)
-            gtk_table_reparent(buttontbl, self.volume_button, 3, 4, 0, 1)
+            gtk_table_reparent(buttontbl, self.play_pause_button, 1, 4, 0, 1)
+            #gtk_table_reparent(buttontbl, self.volume_button, 3, 4, 0, 1)
             gtk_table_reparent(buttontbl, self.rrewind_button, 0, 1, 1, 2)
             gtk_table_reparent(buttontbl, self.rewind_button, 1, 2, 1, 2)
             gtk_table_reparent(buttontbl, self.forward_button, 2, 3, 1, 2)
@@ -470,7 +483,7 @@ class GTK_Main(object):
             main_vbox.pack_start(buttontbl, False, False)
 
             for child in self.main_hbox.get_children():
-                if child != main_vbox and child != self.volume:
+		if child != main_vbox:# and child != self.volume:
                     self.main_hbox.remove(child)
 
         for label in (self.title_label, self.artist_label, self.album_label):
@@ -480,8 +493,8 @@ class GTK_Main(object):
                 xalign = .5
             label.set_alignment(xalign, .5)
 
-        if util.platform == util.MAEMO:
-            self.main_hbox.reorder_child(self.volume, -1)
+        #if util.platform == util.MAEMO:
+        #    self.main_hbox.reorder_child(self.volume, -1)
 
         self.main_window.show_all()
 
@@ -494,6 +507,8 @@ class GTK_Main(object):
             self.app.add_window(window)
         else:
             window = gtk.Window(gtk.WINDOW_TOPLEVEL)
+	
+	self._rotation = FremantleRotation('Panucci', window)
 
         window.set_title('Panucci')
         self.window_icon = util.find_image('panucci.png')
@@ -537,6 +552,7 @@ class GTK_Main(object):
         self.metadata_widget.pack_start(self.album_label, False, False)
         self.title_label = gtk.Label('')
         self.title_label.set_line_wrap(True)
+	self.title_label.set_justify(gtk.JUSTIFY_CENTER)
         self.metadata_widget.pack_start(self.title_label, False, False)
         self.metadata_widget.pack_start(gtk.Label(), True, True)
 
@@ -551,43 +567,47 @@ class GTK_Main(object):
         self.progress_widget.add(self.progress)
 
         # The buttons
-        self.rrewind_button = gtk.Button('')
+	def mkbutton():
+		return hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | \
+				gtk.HILDON_SIZE_THUMB_HEIGHT, 0)
+        self.rrewind_button = mkbutton()
         image(self.rrewind_button, 'media-skip-backward.png')
         self.rrewind_button.connect('clicked', self.seekbutton_callback, -1*long_seek)
-        self.rewind_button = gtk.Button('')
+        self.rewind_button = mkbutton()
         image(self.rewind_button, 'media-seek-backward.png')
         self.rewind_button.connect('clicked', self.seekbutton_callback, -1*short_seek)
-        self.play_pause_button = gtk.Button('')
-        image(self.play_pause_button, gtk.STOCK_OPEN, True)
+        self.play_pause_button = mkbutton()
+        image(self.play_pause_button, 'general_folder', is_name=True)
         self.button_handler_id = self.play_pause_button.connect( 
             'clicked', self.open_file_callback )
-        self.forward_button = gtk.Button('')
+        self.forward_button = mkbutton()
         image(self.forward_button, 'media-seek-forward.png')
         self.forward_button.connect('clicked', self.seekbutton_callback, short_seek)
-        self.fforward_button = gtk.Button('')
+        self.fforward_button = mkbutton()
         image(self.fforward_button, 'media-skip-forward.png')
         self.fforward_button.connect('clicked', self.seekbutton_callback, long_seek)
-        self.bookmarks_button = gtk.Button('')
-        image(self.bookmarks_button, 'bookmark-new.png')
+        self.bookmarks_button = mkbutton()
+        image(self.bookmarks_button, 'filemanager_bookmark', is_name=True)
         self.bookmarks_button.connect('clicked', self.bookmarks_callback)
         self.set_controls_sensitivity(False)
 
         if util.platform == util.MAEMO:
-            self.volume = hildon.VVolumebar()
-            self.volume.set_property('can-focus', False)
-            self.volume.connect('level_changed', self.volume_changed_hildon)
-            self.volume.connect('mute_toggled', self.mute_toggled)
+            #self.volume = hildon.VVolumebar()
+            #self.volume.set_property('can-focus', False)
+            #self.volume.connect('level_changed', self.volume_changed_hildon)
+            #self.volume.connect('mute_toggled', self.mute_toggled)
             window.connect('key-press-event', self.on_key_press)
             window.connect('window-state-event', self.window_state_event)
 
             # Add a button to pop out the volume bar
-            self.volume_button = gtk.ToggleButton('')
-            image(self.volume_button, 'media-speaker.png')
-            self.volume_button.connect('clicked', self.toggle_volumebar)
-            self.volume.connect(
-                'show', lambda x: self.volume_button.set_active(True))
-            self.volume.connect(
-                'hide', lambda x: self.volume_button.set_active(False))
+            #self.volume_button = gtk.ToggleButton('')
+	    #self.volume_button.set_name('HildonButton-thumb')
+            #image2(self.volume_button, 'mediaplayer_volume', is_name=True)
+            #self.volume_button.connect('clicked', self.toggle_volumebar)
+            #self.volume.connect(
+            #    'show', lambda x: self.volume_button.set_active(True))
+            #self.volume.connect(
+            #    'hide', lambda x: self.volume_button.set_active(False))
 
             # Disable focus for all widgets, so we can use the cursor
             # keys + enter to directly control our media player, which
@@ -596,12 +616,13 @@ class GTK_Main(object):
                     self.rrewind_button, self.rewind_button,
                     self.play_pause_button, self.forward_button,
                     self.fforward_button, self.progress, 
-                    self.bookmarks_button, self.volume_button, ):
+		    self.bookmarks_button, ):#self.volume_button, ):
                 w.unset_flags(gtk.CAN_FOCUS)
         else:
-            self.volume = gtk.VolumeButton()
-            self.volume.connect('value-changed', self.volume_changed_gtk)
-            self.volume_button = self.volume
+            #self.volume = gtk.VolumeButton()
+            #self.volume.connect('value-changed', self.volume_changed_gtk)
+            #self.volume_button = self.volume
+	    pass
 
 
         # Generate the basic widget structure
@@ -611,7 +632,7 @@ class GTK_Main(object):
             window.add(self.main_hbox)
         else:
             menu_vbox.pack_end(self.main_hbox, True, True, 6)
-        self.main_hbox.pack_start(self.volume, False, True)
+        #self.main_hbox.pack_start(self.volume, False, True)
 
         # Fill in the widgets in the correct orientation
         self.orientation = self.LANDSCAPE
@@ -620,6 +641,27 @@ class GTK_Main(object):
     def create_app_menu(self):
 	menu = hildon.AppMenu()
 
+	self.button_automatic = gtk.RadioButton()
+	self.button_automatic.connect('clicked', lambda w: self._rotation.set_mode(FremantleRotation.AUTOMATIC))
+	self.button_automatic.set_mode(False)
+	self.button_automatic.set_label(_('Automatic'))
+	menu.add_filter(self.button_automatic)
+
+	self.button_landscape = gtk.RadioButton(self.button_automatic)
+	self.button_landscape.connect('clicked', lambda w: self._rotation.set_mode(FremantleRotation.NEVER))
+	self.button_landscape.set_mode(False)
+	self.button_landscape.set_label(_('Landscape'))
+	menu.add_filter(self.button_landscape)
+
+	self.button_portrait = gtk.RadioButton(self.button_automatic)
+	self.button_portrait.connect('clicked', lambda w: self._rotation.set_mode(FremantleRotation.ALWAYS))
+	self.button_portrait.set_mode(False)
+	self.button_portrait.set_label(_('Portrait'))
+	menu.add_filter(self.button_portrait)
+
+        # Switch to landscape by default (because we don't have a config yet)
+	self.button_landscape.clicked()
+
 	menu_open = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
 	menu_open.set_text(_('Open file'), '')
         menu_open.connect("clicked", self.open_file_callback)
@@ -630,10 +672,10 @@ class GTK_Main(object):
 	menu_queue.connect('clicked', self.queue_file_callback)
         menu.append(menu_queue)
 
-	menu_bookmarks = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
-	menu_bookmarks.set_text(_('Bookmarks'), '')
-	menu_bookmarks.connect('clicked', self.bookmarks_callback)
-        menu.append(menu_bookmarks)
+	#menu_bookmarks = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, hildon.BUTTON_ARRANGEMENT_VERTICAL)
+	#menu_bookmarks.set_text(_('Bookmarks'), '')
+	#menu_bookmarks.connect('clicked', self.bookmarks_callback)
+        #menu.append(menu_bookmarks)
 
 	menu_lock_progress = hildon.CheckButton(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT)
 	menu_lock_progress.set_label('Lock progress bar')
@@ -748,15 +790,14 @@ class GTK_Main(object):
         return settings.progress_locked
 
     def show_about(self, w, win):
-        dialog = gtk.AboutDialog()
-        dialog.set_website(about_website)
-        dialog.set_website_label(about_website)
-        dialog.set_name(about_name)
-        dialog.set_authors(about_authors)
-        dialog.set_comments(about_text)
-        dialog.set_version(app_version)
-        dialog.run()
-        dialog.destroy()
+        authors = ', '.join(about_authors[:-1])
+	authors += ' and ' + about_authors[-1]
+	message = '%s %s for Maemo 5\n%s\n\nBrought to you by %s. Maemo 5 UI by Thomas Perl.\n\nPlease consider supporting the gPodder project if you like Panucci. See http://gpodder.org/donate - thanks!' % (
+			about_name, app_version,
+			about_website, authors)
+	dlg = hildon.hildon_note_new_information(self.main_window, message)
+	dlg.run()
+	dlg.destroy()
 
     def destroy(self, widget):
         player.quit()
@@ -839,9 +880,11 @@ class GTK_Main(object):
 
     def on_key_press(self, widget, event):
         if event.keyval == gtk.keysyms.F7: #plus
-            self.set_volume( min( 1, self.get_volume() + 0.10 ))
+            #self.set_volume( min( 1, self.get_volume() + 0.10 ))
+	    pass # NOT on Maemo 5
         elif event.keyval == gtk.keysyms.F8: #minus
-            self.set_volume( max( 0, self.get_volume() - 0.10 ))
+            #self.set_volume( max( 0, self.get_volume() - 0.10 ))
+	    pass # NOT on Maemo 5
         elif event.keyval == gtk.keysyms.Left: # seek back
             self.rewind_callback(self.rewind_button)
         elif event.keyval == gtk.keysyms.Right: # seek forward
@@ -866,38 +909,41 @@ class GTK_Main(object):
     # The following two functions get and set the
     #   volume from the volume control widgets.
     def get_volume(self):
-        if util.platform == util.MAEMO:
-            return self.volume.get_level()/100.0
-        else:
-            return self.volume.get_value()
+	return 1.0
+        #if util.platform == util.MAEMO:
+        #    return self.volume.get_level()/100.0
+        #else:
+        #    return self.volume.get_value()
 
     def set_volume(self, vol):
         """ vol is a float from 0 to 1 """
         assert 0 <= vol <= 1
-        if util.platform == util.MAEMO:
-            self.volume.set_level(vol*100.0)
-        else:
-            self.volume.set_value(vol)
+        #if util.platform == util.MAEMO:
+        #    self.volume.set_level(vol*100.0)
+        #else:
+        #    self.volume.set_value(vol)
 
     def __set_volume_hide_timer(self, timeout, force_show=False):
-        if force_show or self.volume_button.get_active():
-            self.volume.show()
-            if self.volume_timer_id is not None:
-                gobject.source_remove(self.volume_timer_id)
-
-            self.volume_timer_id = gobject.timeout_add(
-                1000 * timeout, self.__volume_hide_callback )
+        pass
+        #if force_show: #or self.volume_button.get_active():
+        #    self.volume.show()
+        #    if self.volume_timer_id is not None:
+        #        gobject.source_remove(self.volume_timer_id)
+#
+#            self.volume_timer_id = gobject.timeout_add(
+#                1000 * timeout, self.__volume_hide_callback )
 
     def __volume_hide_callback(self):
-        self.volume_timer_id = None
-        self.volume.hide()
+        #self.volume_timer_id = None
+        #self.volume.hide()
         return False
 
     def toggle_volumebar(self, widget=None):
-        if self.volume_timer_id is None:
-            self.__set_volume_hide_timer(5)
-        else:
-           self.__volume_hide_callback()
+	pass
+        #if self.volume_timer_id is None:
+        #    self.__set_volume_hide_timer(5)
+        #else:
+        #   self.__volume_hide_callback()
 
     def volume_changed_gtk(self, widget, new_value=0.5):
         player.volume_level = new_value
@@ -936,11 +982,11 @@ class GTK_Main(object):
 
     def on_player_playing(self):
         self.start_progress_timer()
-        image(self.play_pause_button, 'media-playback-pause.png')
+        image2(self.play_pause_button, 'media-playback-pause.png')
         self.set_controls_sensitivity(True)
 
     def on_player_new_track(self, metadata):
-        image(self.play_pause_button, 'media-playback-start.png')
+        image2(self.play_pause_button, 'media-playback-start.png')
         self.play_pause_button.disconnect(self.button_handler_id)
         self.button_handler_id = self.play_pause_button.connect(
             'clicked', self.on_btn_play_pause_clicked )
@@ -959,13 +1005,13 @@ class GTK_Main(object):
 
     def on_player_paused(self):
         self.stop_progress_timer() # This should save some power
-        image(self.play_pause_button, 'media-playback-start.png')
+        image2(self.play_pause_button, 'media-playback-start.png')
 
     def on_player_end_of_playlist(self):
         self.play_pause_button.disconnect(self.button_handler_id)
         self.button_handler_id = self.play_pause_button.connect(
             'clicked', self.open_file_callback )
-        image(self.play_pause_button, gtk.STOCK_OPEN, True)
+        image2(self.play_pause_button, 'general_folder', is_name=True)
 
     def on_file_queued(self, filepath, success):
         filename = os.path.basename(filepath)
